/* NetRun C++ Wrapper (Public Domain) */
#include <cstdio>
#include <cstdlib>
#include <ctype.h>
#include <cstring>
#include <cmath>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <map>
#include <string>
#include "lib/inc.h"
using namespace std; /* ONLY for 202 examples... */
//using std::cout; using std::cin; using std::endl; /* <- avoid annoyance; Gaddis textbook compatibility */



enum {n_rounds_max=120, n_keys=2*n_rounds_max+2};
int n_rounds=3; // actual rounds to run

// Circular bit rotate left: rotate v left by count bits
unsigned int ROTL(unsigned int v,unsigned int count)
{
	return (v<<count) | (v>>(32-count));
}
unsigned int ROTR(unsigned int v,unsigned int count)
{
	return (v>>count) | (v<<(32-count));
}

// Mix two integers worth of data, A and B, with these keys.
void RC5_rounds(unsigned int &A,unsigned int &B,const unsigned int *keys) {
	A += keys[0];
	B += keys[1];
	for (int round=1;round<=n_rounds;round++) {
		//printf(" round %d: %08x %08x\n", round, A,B);
		A^=B;
		A = ROTL(A,B) + keys[2*round];
		B^=A;
		B = ROTL(B,A) + keys[2*round+1];
	}
}
// Mix two integers worth of data, A and B, with these keys.
void RC5_decrypt(unsigned int &A,unsigned int &B,const unsigned int *keys) {
	for (int round=n_rounds;round>=1;round--) {
		B -= keys[2*round+1];
		B = ROTR(B,A);
		B^=A;
		A -= keys[2*round];
		A = ROTR(A,B);
		A^=B; 
		//printf(" round %d: %08x %08x\n", round, A,B);
	}
	A -= keys[0];
	B -= keys[1];
	
}

/*
 Binary file byte-coincidence histogramming tool.

 Dr. Orion Sky Lawlor, lawlor@alaska.edu, 2013-01-28 (Public Domain)
*/
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>

double clamp(double v) {
	if (v<0.0) return 0.0;
	if (v>1.0) return 1.0;
	else return v;
}

class histo2d {
public:
	enum {n=256}; // byte histogram
	unsigned int data[n][n]; /* [first byte][next byte] */
	int last;
	
	histo2d() { // zero the histogram
		for (int i=0;i<n;i++)
		for (int j=0;j<n;j++)
			data[i][j]=0;
		last=-1;
	}
	
	void add(unsigned char next) {
		if (last!=-1) data[last][next]++;
		last=next;
	}
	
	void save(std::string filename) 
	{ // save to a PPM file
		filename+=".ppm";
		std::ofstream of(filename.c_str(),std::ios_base::binary);
		of<<"P6\n"<<n<<" "<<n<<"\n255\n";
	
	// Find the biggest value (to scale output to bytes)
		unsigned int big=0;
		for (int i=0;i<n;i++)
		for (int j=0;j<n;j++)
			if (data[i][j]>big) big=data[i][j];
		double scale=1.0/big;
	
	// Scale the output to bytes, and write them to disk.
		for (int i=0;i<n;i++)
		for (int j=0;j<n;j++)
		{
			unsigned char out=255*clamp(-0.1*log(data[i][j]*scale));
			
			unsigned char out2=out;
			if (((i&0x1f)==0 || (j&0x1f)==0)) 
				out2*=0.75; // make grid lines
			of.write((char *)&out2,1);
			of.write((char *)&out2,1);
			of.write((char *)&out,1);
		}
	}	
};

// Run some simple tests, encrypting ascending data
void foo(void) {
	srand(0);
	unsigned int keys[n_keys];
	for (int k=0;k<n_keys;k++) keys[k]=rand(); // add key schedule here

	histo2d h;
	for (long test=0;test<8*1000000;test++) {
		unsigned int A=test, B=0; // plaintext
		RC5_rounds(A,B,keys);
		
		h.add(A&0xff); // histogram part of A
	}
	h.save("out");
	std::cout<<"Saved image.\n";
}



